#!/usr/bin/php
<?php
//
// A simple command-line Evernote API demo script that lists all notebooks in
// the user's account and creates a simple test note in the default notebook.
//
// Before running this sample, you must fill in your Evernote developer token.
//
// To run:
//   php EDAMTest.php
//

// Import the classes that we're going to be using

use EDAM\Types\Data, EDAM\Types\Note, EDAM\Types\Resource, EDAM\Types\ResourceAttributes;
use EDAM\Error\EDAMUserException, EDAM\Error\EDAMErrorCode;
use Evernote\Client;

use EDAM\NoteStore;
use EDAM\Types;
ini_set("include_path", ini_get("include_path") . PATH_SEPARATOR . "../../lib" . PATH_SEPARATOR);
require_once 'autoload.php';

require_once 'Evernote/Client.php';

require_once 'packages/Errors/Errors_types.php';
require_once 'packages/Types/Types_types.php';
require_once 'packages/Limits/Limits_constants.php';

// A global exception handler for our program so that error messages all go to the console
function en_exception_handler($exception)
{
	echo "Uncaught " . get_class($exception) . ":\n";
	if ($exception instanceof EDAMUserException) {
		echo "Error code: " . EDAMErrorCode::$__names[$exception->errorCode] . "\n";
		echo "Parameter: " . $exception->parameter . "\n";
	} elseif ($exception instanceof EDAMSystemException) {
		echo "Error code: " . EDAMErrorCode::$__names[$exception->errorCode] . "\n";
		echo "Message: " . $exception->message . "\n";
	} else {
		echo $exception;
	}
}
set_exception_handler('en_exception_handler');

class SynoSimpleXMLElement extends SimpleXMLElement
{
    public function replace(SimpleXMLElement $element) {
        $dom     = dom_import_simplexml($this);
        $import  = $dom->ownerDocument->importNode(
            dom_import_simplexml($element),
            TRUE
        );
        $dom->parentNode->replaceChild($import, $dom);
    }
}

class ContentConverter
{
	public $content;
	private $xml;
	private function convertEnMedia()
	{

	}
	private function convertEnTodo()
	{

	}
	private function traverse($target, $depth)
	{
		for ($i = 0; $i < $depth; ++$i) {
			//echo "	";
		}

		if (!is_object($target)) {
			return;
		}
		
		//echo "[".$target->getName()."]";
		if ($target->getName() == "en-media") {
			//$dom = dom_import_simplexml($target);
			//$dom->parentNode->removeChild($dom);
			$replaceNode = simplexml_load_string("<syno>asdf</syno>");
			$target->replace($replaceNode);
			return;
		}
		if ($target->attributes()) {
			foreach ($target->attributes() as $key => $value) {
			}
		}
		foreach ($target as $node) {
			$this->traverse($node, $depth+1);
		}
	}

	public function convert($note)
	{
		$this->xml = simplexml_load_string(html_entity_decode($note->content), 'SynoSimpleXMLElement');
		$this->traverse($this->xml, 0);
	}

	public function getContent()
	{
		return $this->xml->asXML();
	}

}


class SynoEvernote
{
	public $authToken = "S=s1:U=8f02b:E=14ee39262e3:C=1478be13510:P=185:A=pg162:V=2:H=a82a872e37acceb98e54495566846f04";

    private $client, $userStore, $noteStore;

	public function init() {
		//echo $this->authToken;
		$this->client = new Client(array('token' => $this->authToken));
		$this->userStore = $this->client->getUserStore();
		$this->noteStore = $this->client->getNoteStore();
		$versionOK = $this->userStore->checkVersion(
			"Evernote EDAMTest (PHP)",
			$GLOBALS['EDAM_UserStore_UserStore_CONSTANTS']['EDAM_VERSION_MAJOR'],
			$GLOBALS['EDAM_UserStore_UserStore_CONSTANTS']['EDAM_VERSION_MINOR']
		);
		if ($versionOK == 0) {
			die("The version of SDK is too old.");
		}
	}
	public function listNotebooks() {
		$ret = [];
		$notebooks = $this->noteStore->listNotebooks();
		foreach ($notebooks as $notebook) {
			$ret[$notebook->guid] = $notebook->name;
		}
		return $ret;
	}
	public function getNotebook($notebookGuid) {
		$filter = new NoteStore\NoteFilter();
		$filter->ascending = false;
		//$filter->inactive = true;
		$filter->notebookGuid = $notebookGuid;

		$rspec = new NoteStore\NotesMetadataResultSpec();
		$rspec->includeCreated = True;
		$rspec->includeUpdated = True;
		$rspec->includeDeleted = True;
		$rspec->includeUpdateSequenceNum = True;
		$rspec->includeNotebookGuid = True;
		$rspec->includeTagGuids = True;
		$rspec->includeAttributes = True;

		$queryResult = $this->noteStore->findNotesMetadata($this->authToken, $filter, 0, 300, $rspec);
		
		$ret = [];
		foreach ($queryResult->notes as $note) {
			print_r($note);
			array_push($ret, array(
				'guid' => $note->guid,
				'title' => $note->title,
				'tagGuids' => $note->tagGuids
			));
		}
		return $ret;
	}
	public function debugNote($note, $resources, $tagNames) {
		
		return array(
			'guid' => $note->guid,
			'title' => $note->title,
			'content' => $note->content,
			//'tagGuids' => $note->tagGuids,
			'resources' => $resources,
			'tagNames' => $tagNames
		);
	}
	public function debugHash($hash) {
		$ret = "";
		static $charMapping = "0123456789abcde";
		for ($i = 0; $i < strlen($hash); ++$i) {
			//$ret .= substr($charMapping, )
			$char = substr($hash, $i, 1);
			$asciiCode = ord($char);
			$ret .= substr($charMapping, floor($asciiCode/16), 1);
			$ret .= substr($charMapping, $asciiCode%16, 1);
		}
		echo $ret."\n";
		return $ret;
	}
	public function getNoteTagNames($noteGuid) {
		return $this->noteStore->getNoteTagNames($this->authToken, $noteGuid);
	}
	public function getResource($resGuid) {
		$resource = $this->noteStore->getResource($this->authToken, $resGuid, true, false, true, false);
		return $resource;
	}
	public function getNote($noteGuid) {
		$note = $this->noteStore->getNote($this->authToken, $noteGuid, true, false, false, false);
		$tagNames = $this->getNoteTagNames($noteGuid);
		$resources = [];
		foreach ($note->resources as $res) {
			$fullRes = $this->getResource($res->guid);
			array_push($resources, array(
				'guid' => $res->guid,
				'fileName' => $res->attributes->fileName,
				'data' => array(
					'bodyHash' => $this->debugHash($fullRes->data->bodyHash),
					'size' => $fullRes->data->size,
					'body(Length)' => strlen($fullRes->data->body),
					'body(md5)' => md5($fullRes->data->body)
				) 
			));
		}
		$tmp = new ContentConverter();
		//echo '[Before]'.$tmp->getContent();
		$tmp->convert($note);
		echo '[After]'.$tmp->getContent();
		return $this->debugNote($note, $resources, $tagNames);
		//return $note;
	}

	public function getThumb($noteGuid) {
		echo "https://sandbox.evernote.com/shard/s1/thm/note/".$noteGuid;
	}
}
function SynoShellMode()
{
	$instance = new SynoEvernote();
	$instance->init();
	while ($cmd = readline("SynoEvernote> ")) {
		$argv = explode(" ", $cmd);
		try {
			$result = call_user_func_array(array($instance, $argv[0]), array_slice($argv,1));
		} catch (Exception $e) {
			echo $e->getMessage()."\n";
		}
		print_r($result);
	}
}
SynoShellMode();
exit();

$a = new SynoEvernote();
$a->init();

print_r($a->listNotebooks());
print_r($a->getNotebook("e2eeb4be-3523-43d3-870c-e33e5fab93ac"));
print_r($a->getNote("a886c6c8-2369-42a8-8c9f-94986fd0582a"));
print_r(call_user_func(array($a, 'getNote'), "a886c6c8-2369-42a8-8c9f-94986fd0582a"));
//print_r($argv);
exit();

// Real applications authenticate with Evernote using OAuth, but for the
// purpose of exploring the API, you can get a developer token that allows
// you to access your own Evernote account. To get a developer token, visit
// https://sandbox.evernote.com/api/DeveloperToken.action
$authToken = "S=s1:U=8f02b:E=14ed262abe7:C=1477ab17cf0:P=1cd:A=en-devtoken:V=2:H=a961a2f98809200dfd19cec6f4df3c44";
$authToken = "S=s1:U=8f02b:E=14ee39262e3:C=1478be13510:P=185:A=pg162:V=2:H=a82a872e37acceb98e54495566846f04";
if ($authToken == "your developer token") {
	print "Please fill in your developer token\n";
	print "To get a developer token, visit https://sandbox.evernote.com/api/DeveloperToken.action\n";
	exit(1);
}

// Initial development is performed on our sandbox server. To use the production
// service, change "sandbox.evernote.com" to "www.evernote.com" and replace your
// developer token above with a token from
// https://www.evernote.com/api/DeveloperToken.action
$client = new Client(array('token' => $authToken));

$userStore = $client->getUserStore();

// Connect to the service and check the protocol version
$versionOK =
	$userStore->checkVersion("Evernote EDAMTest (PHP)",
		 $GLOBALS['EDAM_UserStore_UserStore_CONSTANTS']['EDAM_VERSION_MAJOR'],
		 $GLOBALS['EDAM_UserStore_UserStore_CONSTANTS']['EDAM_VERSION_MINOR']);
print "Is my Evernote API version up to date?  " . $versionOK . "\n\n";
if ($versionOK == 0) {
	exit(1);
}

$noteStore = $client->getNoteStore();

// List all of the notebooks in the user's account
$notebooks = $noteStore->listNotebooks();
echo "<pre>";
print "Found " . count($notebooks) . " notebooks\n";
foreach ($notebooks as $notebook) {
	//print_r($notebook);
}

$filter = new NoteStore\NoteFilter();
$filter->ascending = false;
//$filter->inactive = true;
//$filter->notebookGuid = "e2eeb4be-3523-43d3-870c-e33e5fab93ac";
print "filter done\n";

$rspec = new NoteStore\NotesMetadataResultSpec();
$rspec->includeCreated = True;
$rspec->includeUpdated = True;
$rspec->includeDeleted = True;
$rspec->includeUpdateSequenceNum = True;
$rspec->includeNotebookGuid = True;
$rspec->includeTagGuids = True;
$rspec->includeAttributes = True;

print "rspec done\n";

$ourMetadataList = $noteStore->findNotesMetadata($authToken, $filter, 0, 300, $rspec);
print "find done\n";

print_r($ourMetadataList);
function dfs($target, $depth)
{
	//echo $depth;
	for ($i = 0; $i < $depth; ++$i) {
		echo "	";
	}

	if (!is_object($target)) {
		echo "endpoint";
		return;
	}
	
	//echo get_class($target);
	echo "[".$target->getName()."]";
	if ($target->getName() == "en-media") {
		//echo "unset".$target->getName();
		//unset($target);
		//echo $target; 
		$dom = dom_import_simplexml($target);
		$dom->parentNode->removeChild($dom);
		echo "\n";
		return;
	}
	if ($target->attributes()) {
		foreach ($target->attributes() as $key => $value) {
			//echo " ".$key."=".$value;
		}
	}
	echo "\n";
	foreach ($target as $node) {
		dfs($node, $depth+1);
	}

}

exit(0);

foreach ($ourMetadataList->notes as $note)
{
	//print_r($meta);
	echo "Getting [".$note->guid."]\n";
	$result = $noteStore->getNote($authToken, $note->guid, true, false, false, false);
	print_r($result);
	echo $result->title."\n".$result->tagGuids."\n".gettype($result->tagGuids)."\n";
	continue;
	//print "content".$result->content;
	$xml = simplexml_load_string(html_entity_decode($result->content));
	//if ($result->guid == '7bed5ae6-32e1-4187-8688-dd9fcc931b3b') {
		//print_r($xml);
		print $xml->asXML();
		echo "before dfs\n";
		dfs($xml, 0);
		//print_r($xml);
		print $xml->asXML();
	//}
}

exit(0);

print"\nCreating a new note in the default notebook\n\n";

// To create a new note, simply create a new Note object and fill in
// attributes such as the note's title.
$note = new Note();
$note->title = "Test note from EDAMTest.php";

// To include an attachment such as an image in a note, first create a Resource
// for the attachment. At a minimum, the Resource contains the binary attachment
// data, an MD5 hash of the binary data, and the attachment MIME type. It can also
// include attributes such as filename and location.
$filename = "enlogo.png";
$image = fread(fopen($filename, "rb"), filesize($filename));
$hash = md5($image, 1);

$data = new Data();
$data->size = strlen($image);
$data->bodyHash = $hash;
$data->body = $image;

$resource = new Resource();
$resource->mime = "image/png";
$resource->data = $data;
$resource->attributes = new ResourceAttributes();
$resource->attributes->fileName = $filename;

// Now, add the new Resource to the note's list of resources
$note->resources = array( $resource );

// To display the Resource as part of the note's content, include an <en-media>
// tag in the note's ENML content. The en-media tag identifies the corresponding
// Resource using the MD5 hash.
$hashHex = md5($image, 0);

// The content of an Evernote note is represented using Evernote Markup Language
// (ENML). The full ENML specification can be found in the Evernote API Overview
// at http://dev.evernote.com/documentation/cloud/chapters/ENML.php
$note->content =
	'<?xml version="1.0" encoding="UTF-8"?>' .
	'<!DOCTYPE en-note SYSTEM "http://xml.evernote.com/pub/enml2.dtd">' .
	'<en-note>Here is the Evernote logo:<br/>' .
	'<en-media type="image/png" hash="' . $hashHex . '"/>' .
	'</en-note>';

// When note titles are user-generated, it's important to validate them
$len = strlen($note->title);
$min = $GLOBALS['EDAM_Limits_Limits_CONSTANTS']['EDAM_NOTE_TITLE_LEN_MIN'];
$max = $GLOBALS['EDAM_Limits_Limits_CONSTANTS']['EDAM_NOTE_TITLE_LEN_MAX'];
$pattern = '#' . $GLOBALS['EDAM_Limits_Limits_CONSTANTS']['EDAM_NOTE_TITLE_REGEX'] . '#'; // Add PCRE delimiters
if ($len < $min || $len > $max || !preg_match($pattern, $note->title)) {
	print "\nInvalid note title: " . $note->title . '\n\n';
	exit(1);
}

// Finally, send the new note to Evernote using the createNote method
// The new Note object that is returned will contain server-generated
// attributes such as the new note's unique GUID.
$createdNote = $noteStore->createNote($note);

print "Successfully created a new note with GUID: " . $createdNote->guid . "\n";
